import os
import unittest
import shutil
import ConfigParser 
from commons.core.utils.FileUtils import FileUtils
from commons.core.coord.Path import Path
from commons.core.sql.DbMySql import DbMySql
from commons.core.sql.TablePathAdaptator import TablePathAdaptator

EXPECTED_LINE_IN_PROFILES_SEARCH_RESULTS = 16
EXPECTED_LINE_IN_PROFILES_SEARCH_RESULTS_HMMER3 = 1
EXPECTED_LINE_IN_ORF_SEARCH_RESULTS = 50
EXPECTED_LINE_IN_POLYA_SEARCH_RESULTS = 0
EXPECTED_LINE_IN_TR_SEARCH_RESULTS = 12
EXPECTED_LINE_IN_BLRtx_SEARCH_RESULTS = 4
EXPECTED_LINE_IN_BLRx_SEARCH_RESULTS = 4
EXPECTED_LINE_IN_BLRn_SEARCH_RESULTS = 1
EXPECTED_LINE_IN_SSRtrf_SEARCH_RESULTS = 16
CURRENT_DIR = os.getcwd()

class Test_F_TEClassifierStep1(unittest.TestCase):
    
    HOST_NAME = "compute-2-46.local"

    def setUp(self):
        os.chdir(CURRENT_DIR)
        self._prg_opt = "%s/denovo_pipe/TEclassifier.py -s 1 -p" % (os.environ.get( "REPET_PATH" ))   
        self._consensusFile = "ConsensusFile_test.fa"
        self._configFileFull = "%s/commons/pyRepetUnit/hmmer/profilsSearchInTEClassifier/tests/configTestTEClassifierStep1.cfg" % (os.environ.get( "REPET_PATH" ))
        self._host = os.environ.get( "REPET_HOST" )
        self._user = os.environ.get( "REPET_USER" )
        self._passwd = os.environ.get( "REPET_PW" )
        self._dbname = os.environ.get( "REPET_DB" )
        self._port = os.environ.get( "REPET_PORT" )
        self._writeConfigFile(True)
        config = ConfigParser.ConfigParser()
        config.readfp( open(self._configFileFull) )
        self._configFile = os.path.basename( self._configFileFull )
        self._db = DbMySql( cfgFileName = self._configFile )
        self._dirToclean = config.get("project", "project_dir")
        os.mkdir(self._dirToclean)
        self._profilsHmmBank = "datas/hmmbank_test"
        self._project_name = "test_TEClassif"
        self._expectedFile = "datas/hmmpfamOut.align.clean_match.path"
        self._bank = os.path.basename( self._profilsHmmBank )
        self._profilesSearchResults = "%s/HmmProfiles/%s_profiles_%s.align.clean_match.path" % ( self._dirToclean, self._project_name, self._bank )
        self._oRFSearchResultsFile = "ORFs/%s.orf.map" % (self._project_name)
        self._polyASearchResultsFile = "polyA/%s.polyA.set" % (self._project_name)
        self._tRSearchResultsFile = "TRsearch/%s.TR.set" % (self._project_name)
        self._tE_BLRtxResultsFile = "TE_BLRtx/%s_TE_BLRtx_DummyRepbase_nt.fa.align.clean_match.path" % (self._project_name)
        self._tE_BLRxResultsFile = "TE_BLRx/%s_TE_BLRx_DummyRepbase_aa.fa.align.clean_match.path" % (self._project_name)
        self._tE_BLRnResultsFile = "TE_BLRn/%s_TE_BLRn_DummyRepbase_nt.fa.align.clean_match.path" % (self._project_name)
        self._SSRtrfResults = "TRF/%s.SSRtrf.set" % (self._project_name)

    def tearDown(self):
        self._deleteDBtables()
        os.chdir(CURRENT_DIR)
        shutil.rmtree(self._dirToclean)
        os.remove(self._configFileFull)
               
    def testAcceptanceTEClassifierStep1WithHmmProfilesEnabled(self):  
        self._copyDatabanksAndInputFileInProjectDir()
        os.chdir(self._dirToclean)
        self._runTEClassifierStep1() 
        self.assertTrue(self._isTableWasCreated(self._db))
        self.assertEquals(self._countLinesInProfilesSearchResults(self._profilesSearchResults), EXPECTED_LINE_IN_PROFILES_SEARCH_RESULTS)
        lpathResultFromFile = self._readPathResultsFromFileAndFillList()
        lpathResultFromTable = self._readPathResultsFromTableAndFillList() 
        self.assertEquals(lpathResultFromFile, lpathResultFromTable)
        self._checkAnalysisResults()
        self._deleteDBtables()
        self._clean()    
        self._cleanCopyFiles()
        self._checkIfCleanIsCorrect()
        
    def testAcceptanceTEClassifierStep1WithHmmProfilesEnabledWithNew_Hmmer3(self):  
        if not os.environ.has_key("REPET_BANKS"):
            print "REPET_BANKS environment variable missing\n"
        banksDir = os.environ.get("REPET_BANKS")
        if os.path.exists(banksDir + "/" + self._bank):
            self._profilsHmmBank = banksDir + "/" + self._bank
        else : 
            print banksDir + "/" + self._bank + " file missing\n"
        self._copyDatabanksAndInputFileInProjectDir()
        os.chdir(self._dirToclean)
        self._runTEClassifierStep1WithHmmer3() 
        self.assertTrue(self._isTableWasCreated(self._db))
        self.assertEquals(self._countLinesInProfilesSearchResults(self._profilesSearchResults), EXPECTED_LINE_IN_PROFILES_SEARCH_RESULTS_HMMER3)
        lpathResultFromFile = self._readPathResultsFromFileAndFillList()
        lpathResultFromTable = self._readPathResultsFromTableAndFillList() 
        self.assertEquals(lpathResultFromFile, lpathResultFromTable)
        self._checkAnalysisResults()
        self._deleteDBtables()
        self._clean()    
        FileUtils.removeFilesByPattern(self._bank + ".*")
        self._cleanCopyFiles()
        self._checkIfCleanIsCorrect()
        
    def testAcceptanceTEClassifierStep1WithHmmProfilDisabled(self):
        self._writeConfigFile(False)
        self._copyDatabanksAndInputFileInProjectDir()
        os.chdir(self._dirToclean)
        self._runTEClassifierStep1() 
        self.assertFalse(self._isTableWasCreated(self._db))
        self._checkAnalysisResults()
        self._deleteDBtables()
        self._clean()  
        self._cleanCopyFiles()  
        self._checkIfCleanIsCorrect()
        
    def _checkIfCleanIsCorrect(self):
        ListFiles = FileUtils.getFileNamesList(self._dirToclean)
        self.assertEquals(ListFiles, [])

    def _isLineExpectedAndPresentOnlyOneTime(self, booleanDict, regExp, regExpString, booleenName, filledDict):
        if regExp == regExpString:
            if filledDict[booleenName] == False:
                booleanDict[booleenName] = True
                filledDict[booleenName] = True
            else : booleanDict[booleenName] = False

    def _clean(self):
        os.system("rm -r " + self._dirToclean + "/HmmProfiles")
        os.system("rm -r " + self._dirToclean + "/ORFs")
        os.system("rm -r " + self._dirToclean + "/polyA")
        os.system("rm -r " + self._dirToclean + "/TRsearch")
        os.system("rm -r " + self._dirToclean + "/TE_BLRtx")
        os.system("rm -r " + self._dirToclean + "/TE_BLRx")
        os.system("rm -r " + self._dirToclean + "/TE_BLRn")
        os.system("rm -r " + self._dirToclean + "/TRF")
        os.system("rm TEclassif_*")
        os.system("rm *.e*")

    def _deleteDBtables(self):
        self._db.dropTable("%s_Profiles_path" % self._project_name)
        self._db.dropTable("%s_ORF_map" % self._project_name)
        self._db.dropTable("%s_polyA_set" % self._project_name)
        self._db.dropTable("%s_TR_set" % self._project_name)
        self._db.dropTable("%s_SSRtrf_set" % self._project_name)
        self._db.dropTable("%s_TE_BLRtx_path" % self._project_name)
        self._db.dropTable("%s_TE_BLRx_path" % self._project_name)
        self._db.dropTable("%s_TE_BLRn_path" % self._project_name)

    def _checkAnalysisResults(self):
        self.assertEquals(FileUtils.getNbLinesInSingleFile(self._oRFSearchResultsFile), EXPECTED_LINE_IN_ORF_SEARCH_RESULTS)
        self.assertEquals(EXPECTED_LINE_IN_ORF_SEARCH_RESULTS, len(self._queryInDB('select * from %s_ORF_map' % (self._project_name))))
        self.assertEquals(FileUtils.getNbLinesInSingleFile(self._polyASearchResultsFile), EXPECTED_LINE_IN_POLYA_SEARCH_RESULTS)
        self.assertEquals(EXPECTED_LINE_IN_POLYA_SEARCH_RESULTS, len(self._queryInDB('select * from %s_polyA_set' % (self._project_name))))
        self.assertEquals(FileUtils.getNbLinesInSingleFile(self._tRSearchResultsFile), EXPECTED_LINE_IN_TR_SEARCH_RESULTS)
        self.assertEquals(EXPECTED_LINE_IN_TR_SEARCH_RESULTS, len(self._queryInDB('select * from %s_TR_set' % (self._project_name))))
        self.assertEquals(FileUtils.getNbLinesInSingleFile(self._tE_BLRtxResultsFile), EXPECTED_LINE_IN_BLRtx_SEARCH_RESULTS)
        self.assertEquals(EXPECTED_LINE_IN_BLRtx_SEARCH_RESULTS, len(self._queryInDB('select * from %s_TE_BLRtx_path' % (self._project_name))))
        self.assertEquals(FileUtils.getNbLinesInSingleFile(self._tE_BLRxResultsFile), EXPECTED_LINE_IN_BLRx_SEARCH_RESULTS)
        self.assertEquals(EXPECTED_LINE_IN_BLRx_SEARCH_RESULTS, len(self._queryInDB('select * from %s_TE_BLRx_path' % (self._project_name))))
        self.assertEquals(FileUtils.getNbLinesInSingleFile(self._tE_BLRnResultsFile), EXPECTED_LINE_IN_BLRn_SEARCH_RESULTS)
        self.assertEquals(EXPECTED_LINE_IN_BLRn_SEARCH_RESULTS, len(self._queryInDB('select * from %s_TE_BLRn_path' % (self._project_name))))
        self.assertEquals(FileUtils.getNbLinesInSingleFile(self._SSRtrfResults), EXPECTED_LINE_IN_SSRtrf_SEARCH_RESULTS)
        self.assertEquals(EXPECTED_LINE_IN_SSRtrf_SEARCH_RESULTS, len(self._queryInDB('select * from %s_SSRtrf_set' % (self._project_name))))

    def _queryInDB(self, query):
        self._db.execute(query)
        results = self._db.fetchall()
        return results

    def _readPathResultsFromTableAndFillList(self):
        tablePathAdaptatorInstance = TablePathAdaptator (self._db, '%s_Profiles_path' % (self._project_name))
        lPath = tablePathAdaptatorInstance.getListOfAllPaths()
        return lPath

    def _readPathResultsFromFileAndFillList(self):
        pathInstance = Path()
        lPath = []
        f = open( self._profilesSearchResults , "r")
        while pathInstance.read( f ):
            lPath.append(pathInstance)
            pathInstance = Path()
        f.close()
        return lPath
        
    def _countLinesInProfilesSearchResults(self, TEClassifierResults):
        return FileUtils.getNbLinesInSingleFile(TEClassifierResults)

    def _isTableWasCreated(self, db):
        return db.doesTableExist("%s_Profiles_path" % (self._project_name))

    def _runTEClassifierStep1(self):
        cmd = self._prg_opt
        cmd += " %s" % (self._project_name)
        cmd += " -c %s" % (self._configFile)
        cmd += " -i %s" % (self._consensusFile)
        os.system(cmd)
        
    def _runTEClassifierStep1WithHmmer3(self):
        cmd = self._prg_opt
        cmd += " %s" % (self._project_name)
        cmd += " -c %s" % ( self._configFile )
        cmd += " -i %s" % ( self._consensusFile )
        cmd += " -n"
        os.system(cmd)
        
    def _copyDatabanksAndInputFileInProjectDir(self):
        shutil.copy(self._profilsHmmBank, self._dirToclean + "/" + self._bank)
        shutil.copy(self._configFileFull, self._dirToclean + "/" + self._configFile)
        shutil.copy("./datas/DummyRepbase_aa.fa", self._dirToclean + "/" + "DummyRepbase_aa.fa")
        shutil.copy("./datas/DummyRepbase_nt.fa", self._dirToclean + "/" + "DummyRepbase_nt.fa")
        shutil.copy("./datas/" + self._consensusFile, self._dirToclean + "/" + self._consensusFile)

    def _cleanCopyFiles(self):
        os.remove(self._dirToclean + "/" + self._bank)
        os.remove(self._dirToclean + "/" + "DummyRepbase_aa.fa")
        os.remove(self._dirToclean + "/" + "DummyRepbase_nt.fa")
        os.remove(self._dirToclean + "/" + self._consensusFile)
        os.remove(self._dirToclean + "/" + self._configFile)
        
    def _writeConfigFile(self, hmmProfiles):
        f = open(self._configFileFull, "w")
        f.write("[repet_env]\n")
        f.write("repet_user: %s\n" % (self._user))
        f.write("repet_host: %s\n" % (self._host))
        f.write("repet_pw: %s\n" % (self._passwd))
        f.write("repet_db: %s\n" % (self._dbname))
        f.write("repet_port: %s\n" % (self._port))
        f.write("\n")
        f.write("[project]\n")
        f.write("project_name: test_TEClassif\n")
        f.write("project_dir: %s/commons/pyRepetUnit/hmmer/profilsSearchInTEClassifier/tests/repet_results\n" % (os.environ.get( "REPET_PATH" )))
        f.write("\n")
        f.write("[detect_features]\n")
        f.write("tmpDir:\n")
        f.write("clean: yes\n")
        f.write("te_hmmer_evalue: 10\n")
        f.write("te_hmm_profiles: hmmbank_test\n")
        if self.HOST_NAME == os.getenv("HOSTNAME"):
            f.write("resources: test=TRUE\n")
        else:
            f.write("resources: \n") 
        if hmmProfiles:
            f.write("te_hmmer: yes\n")
        else:
            f.write("te_hmmer: no\n")
        f.write("term_rep: yes\n")
        f.write("polyA: yes\n")
        f.write("tand_rep: yes\n")
        f.write("orf: yes\n")
        f.write("TE_BLRtx: yes\n")
        f.write("TE_BLRx: yes\n")
        f.write("TE_nucl_bank: DummyRepbase_nt.fa\n")
        f.write("TE_prot_bank: DummyRepbase_aa.fa\n")
        f.write("HG_BLRn: no\n")
        f.write("blast: wu\n")
        f.write("TE_BLRn: yes\n")
        f.write("rDNA_BLRn: no\n")
        f.write("tRNA_scan: no\n")
        f.close()
        
if __name__ == "__main__":                 
    unittest.main()
